
#include <stdio.h>
#include <string.h>
#include <usb.h>
/*#include <linux/usb.h>*/
#include <fcntl.h>
#include <termios.h>
#include <signal.h>


void print_endpoint(struct usb_endpoint_descriptor *endpoint)
{
  printf("      bEndpointAddress: %02xh\n", endpoint->bEndpointAddress);
  printf("      bmAttributes:     %02xh\n", endpoint->bmAttributes);
  printf("      wMaxPacketSize:   %d\n", endpoint->wMaxPacketSize);
  printf("      bInterval:        %d\n", endpoint->bInterval);
  printf("      bRefresh:         %d\n", endpoint->bRefresh);
  printf("      bSynchAddress:    %d\n", endpoint->bSynchAddress);
}

void print_altsetting(struct usb_interface_descriptor *interface)
{
  int i;

  printf("    bInterfaceNumber:   %d\n", interface->bInterfaceNumber);
  printf("    bAlternateSetting:  %d\n", interface->bAlternateSetting);
  printf("    bNumEndpoints:      %d\n", interface->bNumEndpoints);
  printf("    bInterfaceClass:    %d\n", interface->bInterfaceClass);
  printf("    bInterfaceSubClass: %d\n", interface->bInterfaceSubClass);
  printf("    bInterfaceProtocol: %d\n", interface->bInterfaceProtocol);
  printf("    iInterface:         %d\n", interface->iInterface);

  for (i = 0; i < interface->bNumEndpoints; i++)
    print_endpoint(&interface->endpoint[i]);
}

void print_interface(struct usb_interface *interface)
{
  int i;

  for (i = 0; i < interface->num_altsetting; i++)
    print_altsetting(&interface->altsetting[i]);
}

void print_configuration(struct usb_config_descriptor *config)
{
  int i;

  printf("  wTotalLength:         %d\n", config->wTotalLength);
  printf("  bNumInterfaces:       %d\n", config->bNumInterfaces);
  printf("  bConfigurationValue:  %d\n", config->bConfigurationValue);
  printf("  iConfiguration:       %d\n", config->iConfiguration);
  printf("  bmAttributes:         %02xh\n", config->bmAttributes);
  printf("  MaxPower:             %d\n", config->MaxPower);

  for (i = 0; i < config->bNumInterfaces; i++)
    print_interface(&config->interface[i]);
}

void list_all_devices(void)
{
  struct usb_bus *bus;
  struct usb_device *dev;

  printf("bus/device  idVendor/idProduct\n");
  for (bus = usb_busses; bus; bus = bus->next) {
    for (dev = bus->devices; dev; dev = dev->next) {
      int i;

      printf("%s/%s     %04X/%04X\n", bus->dirname, dev->filename,
        dev->descriptor.idVendor, dev->descriptor.idProduct);

      if (!dev->config) {
        printf("  Couldn't retrieve descriptors\n");
        continue;
      }

      for (i = 0; i < dev->descriptor.bNumConfigurations; i++)
        print_configuration(&dev->config[i]);
    }
  }
}

struct usb_device *find_vendor_product(int vendor, int product)
{
  struct usb_bus *bus;
  struct usb_device *dev;

  for (bus = usb_busses; bus; bus = bus->next) {
    for (dev = bus->devices; dev; dev = dev->next) {
      if((dev->descriptor.idVendor==vendor)&&
         (dev->descriptor.idProduct)==product)
	   return dev;
    }
  }
  return NULL;
}



/**
 * keybord functions for nonwaiting keyboard check
 * code is get from MW tty kbd driver
 */
static int fd;
static struct termios old;

  int kbd_open( void) {
    struct termios new;
    fd = open( "/dev/tty", O_NONBLOCK);
    if (fd < 0) {
      fprintf( stderr, "Error open keyboard");
      return -1;
    }
    if (tcgetattr(fd, &old) < 0) {
      close(fd);
      return -1;
    }
    new = old;  
    new.c_lflag &= ~(ECHO | ICANON | IEXTEN /*| ISIG*/);
    new.c_iflag &= ~(ICRNL | INPCK | ISTRIP | IXON /*| BRKINT*/);
    new.c_cflag &= ~(CSIZE | PARENB); 
    new.c_cflag |= CS8;
    new.c_cc[VMIN] = 0; 
    new.c_cc[VTIME] = 0;
    if(tcsetattr(fd, TCSAFLUSH, &new) < 0) {
      close(fd);
      return -1;
    }
    return 0;
  }
  int kbd_read( unsigned char *chr) {
    int size;    
    size = read( fd, chr, 1); // until ESC key
    return (size==1) ? 1 : 0;
  }
  int kbd_close( void) {
    tcsetattr(fd, TCSANOW, &old);   
    close( fd);
    fd = 0;
    return 0;
  }

/*** Show VID/PID of all devices ***/  
void show_devs( void) {
  struct usb_bus *bus;
  struct usb_device *dev;
  
  usb_init();
  usb_find_busses();
  usb_find_devices();
  
  printf( "\n  Connected devices:\n");
  for (bus = usb_busses; bus; bus = bus->next) {
    printf("BUS #%s\n", bus->dirname);
    for (dev = bus->devices; dev; dev = dev->next) {
      printf("   - x%04X/x%04X (#%s)\n", dev->descriptor.idVendor, dev->descriptor.idProduct, dev->filename);
    }
  }
  printf( "\n");
}

/*** help ***/ 
void show_help( void) {
  printf("     Parameters & Options\n");
  printf(" --help   ... This help.\n");
  printf(" --devs   ... Show VID/PID of all devices connected by USB.\n");
  printf(" -vXXXX   ... Change VID of USB device. XXXX is hexadecimal VID (e.g. -v04A3).\n");
  printf(" -pXXXX   ... Change PID of USB device. XXXX is hexadecimal PID (e.g. -v0002).\n");  
  printf(" -e       ... Enable local echo.\n");
  printf(" -c, -cXX ... Control changing terminal mode in device.\n");
  printf("              XX is hex number of VENDOR REQUEST.(Default is 0x02)\n");
  printf("              \"Start terminal mode\" sends VENDOR request with wValue=1\n");
  printf("              \"Stop terminal mode\" sends VENDOR request with wValue=0\n\n");
}
 
void str2hex( char *str, char cnt, unsigned short *num) {
  unsigned short val = 0;
  while ( cnt) {
    if (( *str<'0' || *str>'9') && (*str<'A' || *str>'F') && (*str<'a' || *str>'f')) break;
    val <<= 4;
    if ( *str>='0' && *str<='9') val += *str-'0';
    else
      if ( *str>='A' && *str<='F') val += *str-'A'+10;
      else
        if ( *str>='a' && *str<='f') val += *str-'a'+10;     
    
    str++; cnt--;
  }
  *num = val;
}


 
unsigned short usb_vid=0xDEAD,usb_pid=0x0101;
unsigned char usb_tmreq=0x02;

#define FLG_ECHO  0x0001
#define FLG_TMODE 0x0002
unsigned short flags = 0;
    
int parse_args( int argc, char *argv[]) {
  unsigned short hlp;
  int i = 1;
  char *opt;
  if ( argc < 2) {
    printf( "  Help for options: ucom --help\n");
  } else {
    while ( i < argc) {
      opt = argv[i];
      if ( *opt++ == '-') {
        if ( *opt == '-') {
          opt++;
          if ( !strcmp( opt, "help")) {
            show_help();
            return 1;
          } else 
            if ( !strcmp( opt, "devs")) {
              show_devs();
              return 1;
            }
        } else {
          switch( *opt++) {
            case 'v': str2hex( opt, 4, &hlp); if ( hlp) usb_vid=hlp; break;
            case 'p': str2hex( opt, 4, &hlp); usb_pid = hlp; break;
            case 'e': flags |= FLG_ECHO; break;
            case 'c': flags |= FLG_TMODE; if ( *opt>='0') { str2hex( opt, 2, &hlp); usb_tmreq = hlp; } break;
          }
        }
      }      
      i++;
    }
  }
  return 0;
}

/**
 * signal
 */  
 char loop = 1;
 void handler_sigterm( int arg)
 {
   loop = 2;   
 }
        
/**
 * main
 */   
 
int main(int argc, char *argv[])
{
  struct usb_device *dev;
  usb_dev_handle *dh;
  unsigned char chr;
  char bufout[80];
  char bufin[4096];
  int ret = 0, size,i;
  
  printf("\nMini USB terminal (through libusb), (C)2003, R.Bartosinski <bartosr@centrum.cz>\n");
  
  if ( parse_args( argc, argv)) {
    return 0;
  }
  
  // init kbd
  if (kbd_open()) {
    fprintf( stderr,"E:open tty keyboard\n");
    return -1;    
  }
  
//register handler for SIGTERM
  if ( signal(SIGTERM,handler_sigterm) == SIG_ERR) {
    fprintf( stderr,"E:Install SIGTERM handler\n");
  }
  
  // init usb
  usb_init();
  usb_find_busses();
  usb_find_devices();  
  dev=find_vendor_product(usb_vid,usb_pid);
  if(!dev) {
    fprintf(stderr,"E: Cannot find usb device with vid=0x%04X pid=0x%04X\n",usb_vid,usb_pid);
    goto err;    
  }  
  
  dh=usb_open(dev);  
  /* usb_claim_interface */  
  /* control terminal mode */
  /* vendor request = 0x02 (value = 1) -> go to the terminal mode */
  /*                       (value = 0) -> stop the terminal mode */
  if ( flags & FLG_TMODE) {
    ret=usb_control_msg(dh, USB_ENDPOINT_OUT | USB_TYPE_VENDOR, usb_tmreq, 1, 0, NULL, 0, 1000); /* start terminal */
    if ( ret) {
      fprintf( stderr,"E: Start Terminal Mode\n");
      goto ende;
    }
  }
  printf("*** Terminal started ***\n\n");                              
  
  while(loop==1) { 
    if ( kbd_read( &chr)) {
      if (chr == 27) loop=0;
        if (chr == 13) size = sprintf(bufout,"\n\r");
        else size = sprintf( bufout, "%c", chr);
      if (flags & FLG_ECHO) {
        fprintf(stdout,"%s",bufout);
        fflush(stdout);
      }      
      usb_bulk_write( dh, USB_ENDPOINT_OUT | 0x02, bufout, size, 100);
      size = 0;
    }
    size = usb_bulk_read( dh, USB_ENDPOINT_IN | 0x02, bufin, 1024, 100); /* stejne je to posilany po znaku ;-) */
    if ( size>0) {
//printf(" %d(%02X) ",size,bufin[0]);
      for( i=0;i<size;i++) {
        fprintf( stdout,"%c",bufin[i]);        
      }
      fflush(stdout);
    }
  }
    
  fflush(stdout);  
  if ( flags & FLG_TMODE) {
    ret=usb_control_msg(dh, USB_ENDPOINT_OUT | USB_TYPE_VENDOR, usb_tmreq, 0, 0, NULL, 0, 1000); /* stop terminal */
    if ( ret) fprintf( stderr,"E: Stop Terminal Mode\n");
  }
ende:
  usb_close(dh);
err:
  kbd_close();
  if ( loop==2) {
    printf("\nBreaked\n");
  }
  printf("Exit...\n");
  return 0;
}
